import os, sys

from .templite import Templite
from .question_list import query
from .md_add_space import update_space
from .md_update_url import update_url

def readme():
    os.path.abspath(__file__)
    ppath = os.path.dirname
    ppath_value = ppath(os.path.abspath(__file__))
    tmp_name = 'standard-readme-django-cn.template'
    tmp_path = os.path.join(ppath_value, tmp_name)
    context = query()
    print('@context')
    with open(tmp_path, encoding='utf-8') as f:
        tmp_content = f.read()
    tpl = Templite(tmp_content, context)
    output_path = os.path.join(os.getcwd(), 'README.md')
    ret_txt = tpl.render(output_path=output_path)
    print("README.md generated in current dir")

help_txt= """
使用 md2 -h 来获取帮助
使用 md2 readme 在当前目录生成 README 文件
使用 md2 update-url filename 来更新md文件的网页链接
使用 md2 update-space filename 来更新md文件的英文字母和汉字的空格
"""

def md():
    args = sys.argv
    if len(args)>1 and args[1] == '-h':
        print(help_txt)
    if len(args)>1 and args[1] == 'readme':
        readme()
    if len(args) >= 3 and args[1] == 'update-url':
        update_url(args[2])
    if len(args) >= 3 and args[1] == 'update-space':
        update_space(args[2])







#! /usr/local/bin/python3
#coding=utf-8

import re


file_path = ''

import sys, os

# file_path = sys.argv[1]
#
# cwd = os.getcwd()
#
# if os.path.exists(file_path):
#     file_path = file_path
# elif os.path.exists(os.path.join(cwd, file_path)):
#     file_path = os.path.join(cwd, file_path)
# else:
#     print("md file not exist")
#     sys.exit(1)


def check_file_path(file_path):
    cwd = os.getcwd()
    if os.path.exists(file_path):
        file_path = file_path
    elif os.path.exists(os.path.join(cwd, file_path)):
        file_path = os.path.join(cwd, file_path)
    else:
        print("md file not exist")
        sys.exit(1)


def update_url(file_path):
    check_file_path(file_path)
    with open(file_path,encoding='utf-8') as f:
        str1 = f.read()

    pattern = "(\s)(http[s]*://[\.a-zA-Z\?/_0-9\-=#]+)(\s*)" # \s可以表示换行符,空格
    str1 = re.sub(pattern, r"\1[\2](\2)\3", str1)


    with open(file_path, encoding='utf-8' ,mode='w+') as f:
        f.write(str1)
    print('file url update success')





#! /usr/local/bin/python3
#coding=utf-8

import sys, os

# file_path = ''
# file_path = sys.argv[1]
#
import sys, os


def check_file_path(file_path):
    cwd = os.getcwd()
    if os.path.exists(file_path):
        file_path = file_path
    elif os.path.exists(os.path.join(cwd, file_path)):
        file_path = os.path.join(cwd, file_path)
    else:
        print("md file not exist")
        sys.exit(1)


def is_alpha(c):
    return ord(c) >= 65 and ord(c) < 97 + 26

def is_ascii(c):
    """
    判断一个字符是不是 ascii 码,比如 asci('a') =  "'a'"
    :param c:
    :return:
    """
    return ascii(c)[1:-1] == c

def is_space(c):
    return c == ' '



def update(str1):
    i = 0
    lens = len(str1)
    letters = []
    while i < lens:
        if is_ascii(str1[i]) and not is_space(str1[i]): # $2
            tmp = [i]
            i += 1
            while i < lens and is_ascii(str1[i]) and not is_space(str1[i]):
                i += 1
            tmp.append(i)
            letters.append(tmp) # $+7
        i += 1 # $+8
    # print(letters)
    ls1 = list(str1) # str1 = '和py啊a'
    space_pos = []
    for item in letters:
        if(item[0] > 0 and not is_space( str1[item[0]-1] )):
            space_pos.append(item[0])
        if (item[1] < lens and not is_space(str1[item[1]])):
            space_pos.append(item[1])
    i = 0
    # print(space_pos)
    while i < len(space_pos):
        space_pos[i] += i
        i += 1
    # print(space_pos)
    for ind in space_pos:
        ls1.insert(ind, ' ')
    ret = ''.join(ls1)
    return ret

def update_space(file_path):
    check_file_path(file_path)
    with open(file_path,encoding='utf-8') as f:
        lines = f.readlines()
    # str1 = '了解python的re 模块,你说呢?'
    lines = [update(line) for line in lines]
    with open(file_path, encoding='utf-8' ,mode='w+') as f:
        f.writelines(lines)
    print('file url update success')






import sys

g_data = {}
g_dependency = {
    'bannerPath': ['banner',True], # bannerPath 依赖 banner
    'license': ['mit',False],# license 依赖 mit,但是依赖关系相反
    'diffYear': ['currentYear',False] # license 依赖 mit,但是依赖关系相反
}

def validate_name(name):
    return len(name) > 2

def validate_maintainer(maintainer):
    return len(maintainer) > 2

def validate_confirm(value):
    return value in ('y', 'n')

def validate_length(value):
    return len(value) > 0

def filter_confirm(value):
    return value == 'y' #  if y return True, else False

questions = [
    {
        'name': 'moduleName',
        'message': '这个项目的名称什么呢(大于两个字符)?',
        'type': 'input',
        'default': '名称待定',
        'validate': validate_name
    }, {
        'name': 'banner',
        'message': '你有一个项目的banner吗?(y/n[default])',
        'type': 'confirm',
        'default': 'n'
    }, {
        'name': 'bannerPath',
        'message': 'banner的路径在哪里(可以是项目里的文件)? 例如: \'img/banner.png\'',
        'type': 'input',
        'when': g_data.get('banner', 'n')
    }, {
        'name': 'badge',
        'message': '你需要一个标准 README 项目徽章吗?(y[default]/n)',
        'type': 'confirm',
        'default': 'y'
    }, {
        'name': 'badges',
        'message': '你需要为更多其他徽章预留位置吗?(y/n[default])',
        'type': 'confirm',
        'default': 'n'
    }, {
        'name': 'longDescription',
        'message': '你是否需要预留一个详细的描述段落?(y[default]/n)',
        'type': 'confirm',
        'default': 'y'
    }, {
        'name': 'security',
        'message': '你是否需要一个优先说明的安全提示段落?(y/n[default])',
        'type': 'confirm',
        'default': 'n'
    }, {
        'name': 'background',
        'message': '你是否需要项目背景描述段落?(y[default]/n)',
        'type': 'confirm',
        'default': 'y'
    }, {
        'name': 'API',
        'message': '你是否需要一个API接口描述段落?(y/n[default])',
        'type': 'confirm',
        'default': 'n'
    }, {
        'name': 'maintainers',
        'message': '谁是主要的维护者?',
        'type': 'input',
        'validate': validate_maintainer
    },
    {
        'name': 'contributingFile',
        'message': '你有 CONTRIBUTING.md(贡献说明文档)吗?(y/n[default])',
        'type': 'confirm',
        'default': 'n'
    },
    {
        'name': 'prs',
        'message': '接受Pr吗?(y[default]/n)',
        'type': 'confirm',
        'default': 'y'
    },
    {
        'name': 'mit',
        'message': '你选择 MIT 开源许可证吗?(y[default]/n)',
        'type': 'confirm',
        'default': 'y'
    }
    , {
        'name': 'license',
        'message': '你想要什么许可证?',
        'type': 'input',
        'validate': validate_length
    },
    {
        'name': 'licensee',
        'message': '许可证的持有人是谁(也许是你的名字)?',
        'type': 'input',
        'validate': validate_length
    }, {
        'name': 'currentYear',
        'message': '许可证的持有时间是今年吗?(y[default]/n)',
        'type': 'confirm',
        'validate': 'y',
    }, {
        'name': 'diffYear',
        'message': '指定一个许可证的持有年份',
        'type': 'input'
    }
]

def depend(name):
    dependency = g_dependency.get(name,[])
    if not dependency: # 没有依赖,返回True
        return True
    if dependency[1]:
        return g_data.get(dependency[0], False)
    else:
        return not g_data.get(dependency[0], False)

def _query():
    for quest in questions:
        if quest['type'] == 'input' and depend(quest['name']) :
            while True:
                value = input(quest['message']) or quest.get('default', '')
                if quest.get('validate', False): # 是否需要检测,默认不需要检测
                    flag = quest['validate'](value)
                    if flag: break # 检测是否合法
                    else:
                        print('输入不合规范')
                else:
                    break
            g_data[quest['name']] = value # 不要放在外面,否则会用value的残值
        elif quest['type'] == 'confirm' and depend(quest['name']) :
            while True:
                value = input(quest['message']) or quest.get('default', 'y')
                flag = validate_confirm(value)
                if flag:
                    value = filter_confirm(value) # 检测是否合法
                    break
                else:
                    print('输入不合规范')
            g_data[quest['name']] = value
    return g_data

def query():
    try:
        data = _query()
        return data
    except KeyboardInterrupt as e:
        print("终止生成模板~")
        sys.exit(0)
    except Exception as e:
        print("系统异常")
        print(e)
        sys.exit(1)
